use Test::More tests => 32;

BEGIN { use_ok( 'GrabEnv', 
		qw( grabenv grabenv_File grabenv_Expect diffenv ) ); }

{
    my %newENV = %ENV;
    $newENV{FOO} = '2';
    my %diffENV = diffenv( \%newENV );
    ok( keys %diffENV == 1 && defined $diffENV{FOO} && $diffENV{FOO} eq '2',
	"diffenv" );
}

for my $opt ( undef, { Startup => 1 } )
{
  loop( grabenv => \&grabenv, $opt );
  loop( grabenv_File => \&grabenv_File, $opt );
  loop( grabenv_Expect => \&grabenv_Expect, $opt );
}


sub loop
{
  my ( $funcname, $grabenv, $opt ) = @_;

  my $pfx = "$funcname";

  while( my ( $key, $value ) = each %$opt )
  {
      $pfx .= " ($key = $value)";
  }

  my %newENV;

  %newENV = ();
  %newENV = $grabenv->( 'csh', 'source testenv.csh', undef, $opt );
  ok( defined $newENV{GRABENV} && $newENV{GRABENV} eq 'csh',
      "$pfx: csh" );

  %newENV = ();
  %newENV = $grabenv->( 'tcsh', 'source testenv.tcsh', undef, $opt );
  ok( defined $newENV{GRABENV} && $newENV{GRABENV} eq 'tcsh',
      "$pfx: tcsh" );


  %newENV = ();
  %newENV = $grabenv->( 'sh', '. testenv.sh', undef, $opt );
  ok( defined $newENV{GRABENV} && $newENV{GRABENV} eq 'sh',
      "$pfx: sh" );

  # on some platforms (Linux, ksh93r) turning on emacs mode in the user's .profile
  # and then turning it off as we do here will cause a segv.  I'm not sure how to
  # a) get the ksh version and b) figure out which sequence will cause the segv.
  # hence, just check for one and skip the test if it exists.
  eval {
      %newENV = ();
      %newENV = $grabenv->( 'ksh', '. testenv.ksh', undef, $opt );
  };
 SKIP: {
      # the bug is due to a SegV, but Expect will return an error of
      # either EOF or a signal 11, depending upon timing
      skip("triggered fatal bug in ksh ($@) ; skipping test", 2)
	if $@ && ( $@ =~ /status 11/ || $@ =~ /EOF/);

      ok( defined $newENV{GRABENV} && $newENV{GRABENV} eq 'ksh',
	  "$pfx: ksh" );


      %newENV = ();
      my %oldENV = $grabenv->( 'ksh', 'export FOO=1', undef, $opt );
      %newENV = $grabenv->( 'ksh', 'export FOO=2', undef, $opt );
      my %diffENV = diffenv( \%newENV, \%oldENV );
      ok( keys %diffENV == 1 && defined $diffENV{FOO} && $diffENV{FOO} eq '2',
	  "$pfx: diff envs" );


  }

}

